/**
 * author: mshaik
 * date: 05/20/2015
 */
jQuery.sap.require("sap.sf.learning.ui5.model.I18nModel");
jQuery.sap.require("sap.sf.learning.ui5.model.DataModel");
jQuery.sap.require("sap.sf.learning.instructor.service.InstructorServices");
jQuery.sap.require("sap.sf.learning.instructor.InstructorRouter");
jQuery.sap.require("sap.sf.learning.instructor.util.Formatter");
jQuery.sap.declare("sap.sf.learning.instructor.Component");

sap.ui.core.UIComponent.extend("sap.sf.learning.instructor.Component", {
	metadata : {
		name : "Instructor App",
		version : "1.0",
		includes : [],
		dependencies : {
			libs : ["sap.m", "sap.ui.layout"],
			components : []
		},

		rootView : "sap.sf.learning.instructor.view.Instructor",

		config : {
			resourceBundles : ["Instructor"]
		},

		routing : {
			config : {
				routerClass: "sap.sf.learning.instructor.InstructorRouter",
				viewType : "XML",
				viewPath : "sap.sf.learning.instructor.view",
				clearTarget : false,
				transition: "slide",
				targetControl : "splitApp",
				targetAggregation: "pages"
			},
			routes : [
					{
						pattern : "ScheduleStatus/{scheduleStatus}",
						name : "HOME",
						view : "InstructorSOList"
					},
					{
						pattern : "ScheduleOffering/{scheduleID}&itemTitle={itemTitle}&scheduleStatus={scheduleStatus}",
						name : "SO_DETAILS",
						view : "ScheduleOfferingDetails"
					},
					{
						pattern : "ScheduleOffering/{scheduleID}/Segment/{segmentID}",
						name : "SEGMENT_DETAILS",
						view : "SegmentDetails"
					},
					{
						pattern : "ScheduleOffering/{scheduleID}/RecordLearning",
						name : "RECORD_LEARNING",
						view : "RecordLearning"
					 },
					 {
						pattern : "NoResults",
						name : "NO_RESULTS",
						view : "NoResults"
					 }
		    ]
		}
	},

	init : function() {
		sap.ui.core.UIComponent.prototype.init.apply(this, arguments);
		
		var appParamsModel = new sap.ui.model.json.JSONModel(window.APP_PARAMS);
		sap.ui.getCore().setModel(appParamsModel, 'appParams');
		this.setModel(appParamsModel, 'appParamsModel');

		var mConfig = this.getMetadata().getConfig();
		
		// Always use absolute paths relative to our own component
		var oRootPath = jQuery.sap.getModulePath("sap.sf.learning.instructor");

		// Set i18n model
		var i18nModel = new sap.sf.learning.ui5.model.I18nModel(mConfig.resourceBundles);
		sap.ui.getCore().setModel(i18nModel, "i18nModel"); 
		this.setModel(i18nModel, "i18nModel");

		// Create and set domain model to the component
		var userPrefModel = new sap.sf.learning.ui5.model.DataModel();
		userPrefModel.loadData(this.getService().getUserPreferences());
		this.setModel(userPrefModel, "userPrefModel");

		// Set device model
		var oDeviceModel = new sap.ui.model.json.JSONModel({
			isTouch : sap.ui.Device.support.touch,
			isNoTouch : !sap.ui.Device.support.touch,
			isPhone : sap.ui.Device.system.phone,
			isNoPhone : !sap.ui.Device.system.phone,
			listMode : (sap.ui.Device.system.phone ? "None" : "SingleSelectMaster"),
			listItemType : (sap.ui.Device.system.phone ? "Active" : "Inactive"),
			browserName : sap.ui.Device.browser.name
		});
		oDeviceModel.setDefaultBindingMode("OneWay");
		sap.ui.getCore().setModel(oDeviceModel, "deviceModel"); 
		this.setModel(oDeviceModel, "deviceModel");
		
		// Set view properties model
		var oViewPropertiesModel = new sap.ui.model.json.JSONModel({
			fullScreenIcon : "sap-icon://full-screen",
			fullScreenTooltip: i18nModel.getLabel("label.FullScreen"),
			scheduleOfferingsMode: 'CURRENT',
			scheduleOfferingSelectedTab: 'SCHEDULE',
			reloadStudentsTab:false
		});
		this.setModel(oViewPropertiesModel, "viewPropertiesModel");
		
		// Initialize router
		this.getRouter().initialize();
	},
	
	createContent : function() {
        var oViewData = {
            component : this
        };

        return sap.ui.view({
            viewName : "sap.sf.learning.instructor.view.Main",
            type : sap.ui.core.mvc.ViewType.XML,
            viewData : oViewData
        });
    },
    
    getService: function () {
    	return sap.sf.learning.instructor.service.InstructorServices.getInstance();
    },
	
	getEventBus : function () {
		return sap.ui.getCore().getEventBus();
	}
});