/**
 * author: mshaik
 * date: 05/20/2015
 */
jQuery.sap.declare("sap.sf.learning.instructor.service.InstructorServices");
sap.ui.define(['jquery.sap.global', 'sap/ui/base/Object', 'sap/sf/learning/ui5/model/DataModel'], 
function(jQuery, BaseObject, DataModel) {
	"use strict";

	var InstructorServices = BaseObject.extend("sap.sf.learning.instructor.service.InstructorServices", {
		constructor : function() {
			if (InstructorServices.instance) {
				return InstructorServices.instance;
			}
			InstructorServices.instance = this;

			BaseObject.apply(this, arguments);
			this.init();
		}
	});

	InstructorServices.getInstance = function() {
		if (InstructorServices.instance)
			return InstructorServices.instance;

		InstructorServices.instance = new InstructorServices();
		return InstructorServices.instance;
	};

	InstructorServices.prototype.init = function() {
		var appParamsModelData = sap.ui.getCore().getModel('appParams').getData();
		this._getUserPreferences = {
			type : 'GET',
			url : appParamsModelData.contextPath + '/user/api/v2/i18n/currentUserPreferences'
		};

		this._getScheduleOfferings = {
			type : 'GET',
			url : appParamsModelData.contextPath + '/instructor/api/v1/current-user/current-schedules',
            error: DataModel.handleError
		};

		this._getPastSchedulePastOfferings = {
			type : 'GET',
			url : appParamsModelData.contextPath + '/instructor/api/v1/current-user/past-schedules',
            error: DataModel.handleError
		};
		this._getScheduleOfferingSummary = {
			type : 'GET',
			url : appParamsModelData.contextPath + '/instructor/api/v1/schedules/{scheduleID}/~summary',
            error: DataModel.handleError
		};
		this._getSegmentDetails = {
			type : 'GET',
			url : appParamsModelData.contextPath + '/instructor/api/v1/schedules/{scheduleID}/segments/{segmentID}',
            error: DataModel.handleError
		};		
		this._getStudents = {
    		type : 'GET',
    		url : appParamsModelData.contextPath + '/instructor/api/v1/schedules/{scheduleID}/students',
            error: DataModel.handleError
    	};
		this._getRecordLearningData = {
    		type : 'GET',
    		url : appParamsModelData.contextPath + '/instructor/api/v1/schedules/{scheduleID}/record-learning/~data',
            error: DataModel.handleError
    	};
		this._recordLearning = {
    		type : 'POST',
    		url : appParamsModelData.contextPath + '/instructor/api/v1/schedules/{scheduleID}/record-learning',
            error: DataModel.handleError,
            contentType: "application/json; charset=utf-8",
            dataType: "json"
		};
		this._hasAccessToSendEvaluations = {
    		type : 'GET',
    		url : appParamsModelData.contextPath + '/instructor/api/v1/schedules/{scheduleID}/~has-access-to-send-evaluations',
            error: DataModel.handleError
		};
		this._sendEvaluations = {
    		type : 'POST',
    		url : appParamsModelData.contextPath + '/instructor/api/v1/schedules/{scheduleID}/~send-evaluations',
            error: DataModel.handleError
		};
		this._getSchedSegsStudentsAttendanceCount = {
    		type : 'GET',
    		url : appParamsModelData.contextPath + '/instructor/api/v1/schedules/{scheduleID}/~students-attendance-count',
            error: DataModel.handleError,
		};
		this._getScheduleSegmentStudentsForAttendance = {
			type : 'GET',
			url : appParamsModelData.contextPath + '/instructor/api/v1/schedules/{scheduleID}/segments/{segmentID}/students/~attendance',
            error: DataModel.handleError
		};	
		this._getScheduleSegments = {
    		type : 'GET',
    		url : appParamsModelData.contextPath + '/instructor/api/v1/schedules/{scheduleID}/segments',
            error: DataModel.handleError
    	};
		this._hasAccessToRecordLearning = {
    		type : 'GET',
    		url : appParamsModelData.contextPath + '/instructor/api/v1/schedules/{scheduleID}/~has-access-to-record-learning',
            error: DataModel.handleError
		};
		this._recordSegmentAttendance = {
    		type : 'POST',
    		url : appParamsModelData.contextPath + '/instructor/api/v1/schedules/{scheduleID}/segments/{segmentID}/record-attendance',
            error: DataModel.handleError,
            contentType: "application/json; charset=utf-8",
            dataType: "json"
		};
	};

	InstructorServices.prototype.getUserPreferences = function() {
		return this._getUserPreferences;
	};

	InstructorServices.prototype.getScheduleOfferings = function() {
		return this._getScheduleOfferings;
	};

	InstructorServices.prototype.getPastScheduleOfferings = function() {
		return this._getPastSchedulePastOfferings;
	};

	InstructorServices.prototype.getScheduleOfferingSummary = function(sScheduleID) {
		var oAjax = jQuery.extend({}, this._getScheduleOfferingSummary);
		oAjax.url = oAjax.url.replace("{scheduleID}", sScheduleID);
		return oAjax;
	};

	InstructorServices.prototype.getSegmentDetails = function(sScheduleID, sSegmentID) {
		var oAjax = jQuery.extend({}, this._getSegmentDetails);
		oAjax.url = oAjax.url.replace("{scheduleID}", sScheduleID).replace("{segmentID}", sSegmentID);
		return oAjax;
	};
	InstructorServices.prototype.getScheduleSegments = function(sScheduleID){
		var oAjax = jQuery.extend({}, this._getScheduleSegments);
		oAjax.url = oAjax.url.replace("{scheduleID}", sScheduleID);
		return oAjax;
	}; 
	InstructorServices.prototype.getStudents = function(sScheduleID,sScheduleStatus){
		var oAjax = jQuery.extend({}, this._getStudents);
		oAjax.url = oAjax.url.replace("{scheduleID}", sScheduleID);
		oAjax.data = {scheduleStatus:sScheduleStatus};
		return oAjax;
	}; 
	
	InstructorServices.prototype.getRecordLearningData = function(sScheduleID){
		var oAjax = jQuery.extend({}, this._getRecordLearningData);
		oAjax.url = oAjax.url.replace("{scheduleID}", sScheduleID);
		return oAjax;
	}; 
	
	InstructorServices.prototype.recordLearning = function(sScheduleID,aSelectedStudents){
		var oAjax = jQuery.extend({}, this._recordLearning);
		oAjax.url = oAjax.url.replace("{scheduleID}", sScheduleID);
		oAjax.data =  JSON.stringify(aSelectedStudents);
		return oAjax;
	}; 
	
	InstructorServices.prototype.hasAccessToSendEvaluations = function(sScheduleID){
		var oAjax = jQuery.extend({}, this._hasAccessToSendEvaluations);
		oAjax.url = oAjax.url.replace("{scheduleID}", sScheduleID);
		return oAjax;
	}; 
	
	InstructorServices.prototype.sendEvaluations = function(sScheduleID){
		var oAjax = jQuery.extend({}, this._sendEvaluations);
		oAjax.url = oAjax.url.replace("{scheduleID}", sScheduleID);
		return oAjax;
	}; 
	
	InstructorServices.prototype.getSchedSegsStudentsAttendanceCount = function(sScheduleID,aSelectedStudents) {
		var oAjax = jQuery.extend({}, this._getSchedSegsStudentsAttendanceCount);
		oAjax.url = oAjax.url.replace("{scheduleID}", sScheduleID);
		oAjax.data = {selectedStudentIds:aSelectedStudents};
		return oAjax;
	};

	InstructorServices.prototype.getScheduleSegmentStudentsForAttendance = function(sScheduleID,sSegmentID) {
		var oAjax = jQuery.extend({}, this._getScheduleSegmentStudentsForAttendance);
		oAjax.url = oAjax.url.replace("{scheduleID}", sScheduleID).replace("{segmentID}", sSegmentID);
		return oAjax;
	};
	
	InstructorServices.prototype.hasAccessToRecordLearning = function(sScheduleID){
		var oAjax = jQuery.extend({}, this._hasAccessToRecordLearning);
		oAjax.url = oAjax.url.replace("{scheduleID}", sScheduleID);
		return oAjax;
	};
	
	InstructorServices.prototype.recordSegmentAttendance = function(sScheduleID,sSegmentID,aSchedSegAttendanceVos){
		var oAjax = jQuery.extend({}, this._recordSegmentAttendance);
		oAjax.url = oAjax.url.replace("{scheduleID}", sScheduleID).replace("{segmentID}", sSegmentID);
		oAjax.data =  JSON.stringify(aSchedSegAttendanceVos);
		return oAjax;
	}; 

	return InstructorServices;
}, true);