sap.ui.controller("sap.sf.learning.instructor.view.InstructorSOList", {

	onInit : function() {	
		this._soListCache = {};
		var oModel = new sap.sf.learning.ui5.model.DataModel();
		this.getView().setModel(oModel);		
		var self = this;
		this.getEventBus().subscribe("updateRecordedStudent", function(channel,oEvent,data) {
			if(self.getScheduleOfferingsMode() === 'PAST'){
				var oModel = self.getView().getModel();
				var oList = self.getCurrentSOList();
				var sPath = oList.getSelectedItem().getBindingContext().getPath();
				var iRecordedStudentCount = (data.recordedStudentCount + oModel.getProperty(sPath+'/completionsCount'));
				oModel.setProperty(sPath+'/completionsCount',iRecordedStudentCount);
				oModel.setProperty(sPath+'/localizedData/formattedCompletions',iRecordedStudentCount);
			}
		});
		this.getRouter().attachRoutePatternMatched(this.onRoutePatternMatched, this);
	},
				
	scheduleTabBarChanged: function(oEvent) {
		var sSelectedKey = oEvent.getParameter("selectedKey");
		this.getRouter().navTo("HOME", {			
			scheduleStatus : sSelectedKey
		});
	},
	
	_loadMasterListItems : function(sOfferingsType) {
		this.oInitialLoadFinishedDeferred = jQuery.Deferred();
		var oList = this.getCurrentSOList();
		oList.setBusy(true);
		var self = this;
		var oComponent = this.getComponent();

		if (this._soListCache.hasOwnProperty(sOfferingsType)) {
			this.getView().getModel().setData(this._soListCache[sOfferingsType]);
			oList.setBusy(false);
			self.oInitialLoadFinishedDeferred.resolve();
		} else {
			jQuery.ajax((sOfferingsType === 'PAST' ? oComponent.getService().getPastScheduleOfferings() : oComponent.getService().getScheduleOfferings())).done(function(oResponse) {
				var aOfferingsList = sap.sf.learning.ui5.model.DataModel.getDataFromRestOperationStatusVOX(oResponse);
				self.getView().getModel().setData(aOfferingsList);
				self._soListCache[sOfferingsType] = aOfferingsList;
				oList.setBusy(false);
				self.oInitialLoadFinishedDeferred.resolve();
			});
		}
	},
	
	getCurrentSOList: function(){ 
		var mode= this.getScheduleOfferingsMode();
		if(mode == "FUTURE") {
			mode = "CURRENT";
		}
		var oList = this.byId("listContent_" + mode);
		return oList;
	},

	getSOGroupHeader : function(oGroup) {
		var i18nModel = this.getComponent().getModel('i18nModel');
		var selectedTab = this.getScheduleOfferingsMode();
		return new sap.m.GroupHeaderListItem({
			title : (oGroup.key === 'CURRENT' ? i18nModel.getLabel('label.InProgressClasses') : i18nModel.getLabel('label.FutureClasses')),
			upperCase : false,
			visible : (selectedTab === 'CURRENT')
		});
	},

	showEnrollmentProgressBar : function(minimumEnrollment, maximumEnrollment, scheduleStatus) {
		return (scheduleStatus === 'FUTURE' && (minimumEnrollment || maximumEnrollment)) > 0;
	},

	getRegEnrollmentTextState : function(registrationsCount, minimumEnrollment) {
		var selectedTab = this.getScheduleOfferingsMode();
		return (selectedTab === 'PAST' || (minimumEnrollment && registrationsCount >= minimumEnrollment) || (!minimumEnrollment && registrationsCount !== 0))? "None": "Error";
	},

	getProgressBarColor : function(registrationsCount, minimumEnrollment) {
		return ((registrationsCount >= minimumEnrollment && registrationsCount != 0) ? "Good" : "Error");
	},

	getProgressBarMaxValue : function(registrationsCount, maximumEnrollment) {
		return (registrationsCount > maximumEnrollment ? registrationsCount : maximumEnrollment);
	},

	getRegistrationEnrollmentText : function(registrationsCount, minimumEnrollment, maximumEnrollment, fmtRegistrationsCount, fmtMinimumEnrollment, fmtMaximumEnrollment, fmtCompletionsCount) {
		var i18nModel = this.getComponent().getModel('i18nModel');
		var progressBarText = "";
		var selectedTab = this.getScheduleOfferingsMode();

		if (selectedTab === 'CURRENT') {
			//when max enrollment is set.
			if (maximumEnrollment) {
				progressBarText = i18nModel.getLabel('label.StudentsRegisteredOfMax', [fmtRegistrationsCount, fmtMaximumEnrollment])
			} else if (minimumEnrollment) {
				//when max enrollment is not set and min enrollment is set.
				progressBarText = i18nModel.getLabel('label.StudentsRegisteredNoMaxLimit', [fmtRegistrationsCount, fmtMinimumEnrollment]);
			} else {
				//when min and max enrollment are not set.
				progressBarText = i18nModel.getLabel('label.StudentsRegistered', [fmtRegistrationsCount]);
			}
		} else {
			progressBarText = i18nModel.getLabel('label.StudentsRecordLearningCompleted', [fmtCompletionsCount]);
		}
		
		return progressBarText;
	},

	getSOListTitle: function (length) {
		var i18nModel = this.getComponent().getModel('i18nModel');
		return i18nModel.getLabel('label.MyOfferings', [length || 0]);
	},

	onScheduleOfferingSelect : function(oEvent) {
		this.setScheduleOfferingSelectedTab('SCHEDULE');
		var oListItem = (oEvent.getParameter("listItem") || oEvent.getSource());
		this._navToListItem(oListItem);
	},

	onRoutePatternMatched : function(oEvent) {
		var sRoute = oEvent.getParameter("name");
		var oParams = oEvent.getParameter("arguments");
		
		this._scheduleStatus = oParams.scheduleStatus;
		this.setScheduleOfferingsMode(this._scheduleStatus);
		this.getView().byId('idIconTabBar').setSelectedKey(this._scheduleStatus).focus();
		
		if (sRoute !== "HOME")
			return;
		
		this._loadMasterListItems(this.getScheduleOfferingsMode());
		var oList = this.getCurrentSOList();
		oList.removeSelections(true);
		var pageTitleString = this.getComponent().getModel("i18nModel").getLabel('label.MyClasses');
		document.title = pageTitleString;
		if(!sap.sf.learning.instructor.util.Formatter.isWebApp())  {
			var escapedJsonParameters = escape(pageTitleString);
			var url = "ios://navigationUpdate?pageTitle=" + escapedJsonParameters;
			window.location.href = url;
		}
	},
	
	getNoDataTextForList : function(scheduleOfferingSelectedTab) {
		var i18nModel = this.getComponent().getModel('i18nModel');
		return i18nModel.getLabel(scheduleOfferingSelectedTab === 'CURRENT' ? 'label.NoScheduledClasses' : 'label.NoPastClasses');
	},

	_navToListItem : function(oListItem) {
		var oCtx = oListItem.getBindingContext();
		var oModel = this.getView().getModel();
		var sScheduleStatus = oModel.getProperty(oCtx.getPath()+'/scheduleStatus');
		this.setScheduleOfferingsMode(sScheduleStatus);
		this.getComponent().getModel('viewPropertiesModel').setProperty('/scheduleID', oCtx.getProperty('scheduleID'));
		this.getRouter().navTo("SO_DETAILS", {			
			scheduleID : encodeURIComponent(oCtx.getProperty("scheduleID")), itemTitle : encodeURIComponent(oCtx.getProperty("scheduleDesc")), scheduleStatus : encodeURIComponent(oCtx.getProperty("scheduleStatus"))
		});
	},

	waitForInitialListLoading : function(fnToExecute) {
		jQuery.when(this.oInitialLoadFinishedDeferred).then(jQuery.proxy(fnToExecute, this));
	},
	
	setScheduleOfferingsMode : function(scheduleStatus){		
		var oViewPropertiesModel = this.getComponent().getModel('viewPropertiesModel');	
		oViewPropertiesModel.setProperty('/scheduleOfferingsMode', scheduleStatus);
	},
	
	getScheduleOfferingsMode : function(){
		return this.getComponent().getModel('viewPropertiesModel').getProperty('/scheduleOfferingsMode');
	},
	
	setScheduleOfferingSelectedTab:function(selectedTab){
		var oViewPropertiesModel = this.getComponent().getModel('viewPropertiesModel');
		oViewPropertiesModel.setProperty('/scheduleOfferingSelectedTab',selectedTab);
	},
		
	showRegistrationStatusText: function(scheduleStatus) {
		return (scheduleStatus != 'CURRENT');
	},
			
	getComponent : function() {
		return sap.ui.component(sap.ui.core.Component.getOwnerIdFor(this.getView()));
	},

	getEventBus : function() {
		return this.getComponent().getEventBus();
	},

	getRouter : function() {
		return sap.ui.core.UIComponent.getRouterFor(this);
	}
});
