sap.ui.controller("sap.sf.learning.instructor.view.RecordLearning", {
	
	onInit: function() {
		this.getRouter().attachRouteMatched(this.onRouteMatched, this);
	},
	onRouteMatched: function (oEvent) {
		var sRoute = oEvent.getParameter("name");
		var oParams = oEvent.getParameter("arguments");
		this._scheduleID = oParams.scheduleID;
		if (sRoute !== "RECORD_LEARNING")
			return;
		
		var pageTitleString = this.getComponent().getModel("i18nModel").getLabel('label.RecordLearningCompletions');
		document.title = pageTitleString;
	},
	onNavBack: function (oEvent) {
		this.getRouter().myNavBack("SO_DETAILS", {
			scheduleID: this._scheduleID
		});
	},
	onEditComment: function(oEvent){
		var oModel = this.getComponent().getModel("recordLearningModel");
		var sPath = oEvent.getSource().getBindingContext("recordLearningModel").getPath()+'/comment';
		var sComment =oModel.getProperty(sPath);
		var i18nModel = this.getComponent().getModel("i18nModel");
		var sCommentLabel = i18nModel.getLabel("label.EditComments");
		if(!sComment || sComment == ""){
			sCommentLabel  = i18nModel.getLabel("label.AddComments");
		}
		var dialog = new sap.m.Dialog({
	        title: sCommentLabel,
	        content: new sap.m.TextArea({
	        	width:'100%'
	        }).bindProperty('value',{path:'recordLearningModel>'+sPath}),
	        beginButton:new sap.m.Button({
	          text: '{i18nModel>label.Cancel}',
	          press: function () {
	        	oModel.setProperty(sPath,sComment?sComment:'');
	            dialog.close();
	          }
	        }),
	        endButton:new sap.m.Button({
		          text: '{i18nModel>label.Submit}',
		          type: sap.m.ButtonType.Emphasized,
		          press: function () {
		            dialog.close();
		          }
		        }),
	        afterClose: function() {
	          dialog.destroy();
	        }
	      });
		this.getView().addDependent(dialog);
	    dialog.open();
	},
	onDeleteRecord: function(oEvent){
		var sPath = oEvent.getSource().getBindingContext("recordLearningModel").getPath(); 
		this.getComponent().getModel("recordLearningModel").deleteProperty(sPath);
	},
	onResetRecordLearningForm: function(oEvent){
		this.getComponent().getModel("recordLearningModel").reset();
	},
	onSelectCompletionStatus: function(oEvent){
		var oModel = this.getComponent().getModel("recordLearningModel");
		var sSelectedKey = oEvent.getSource().getSelectedKey();
		var bProvideCredit = oEvent.getSource().getSelectedItem().getBindingContext("recordLearningModel").getProperty('provideCredit');
		var sPath = oEvent.getSource().getBindingContext("recordLearningModel").getPath(); 
		oModel.setProperty(sPath+'/completionStatus',sSelectedKey);
		if(!bProvideCredit){
			oModel.setProperty(sPath+'/creditHours','');
			oModel.setProperty(sPath+'/cpeHours','');
		}
	},
	onSelectGradingOption : function(oEvent) {
		var oModel = this.getComponent().getModel("recordLearningModel");
		var sSelectedKey = oEvent.getSource().getSelectedKey();
		var sPath = oEvent.getSource().getBindingContext("recordLearningModel").getPath(); 
		oModel.setProperty(sPath+'/grade',sSelectedKey);
	},
	onConfirmRecordLearning: function(oEvent){
		var i18nModel = this.getComponent().getModel("i18nModel");
		var self = this;
		var bCompact = !!this.getView().$().closest(".sapUiSizeCompact").length;
		sap.m.MessageBox.confirm(i18nModel.getLabel("label.RecordLearning.Confirmation"), {
        	title : '{i18nModel>label.RecordLearning}',
        	onClose : function(oAction) {
        		if (oAction == 'OK') {
        				self.submitRecordLearningData();
        			}
        		}
		});
	},
	getStatusColumnWidth : function(sBrowserName){
		if(sBrowserName==='ff'){
			return "7rem";
		}
		return "6rem";
	},
	getStudentColumnWidth : function(sBrowserName){
		if(sBrowserName === 'cr'){
			return "14rem";
		}
		return "18rem";
	},
	getDeleteColumnWidth : function(sBrowserName){
		if(sBrowserName === 'cr'){
			return "0.05rem";
		}
		return "1.3rem";
	},
	getGradeColumnWidth : function(iGradingOption,sBrowserName){
		if(iGradingOption !== 2){
			return "auto";
		}else if(sBrowserName === 'ff'){
			return "8rem";
		}
		return "7rem";
	},
	submitRecordLearningData:function(){
		this.getView().setBusy(true);
		var aStudents = [];
		var self = this;
		var oRecordLearningModel = this.getComponent().getModel("recordLearningModel");
		var aSelectedStudents = oRecordLearningModel.getProperty('/selectedStudents');
		var sScheduleID = oRecordLearningModel.getProperty('/scheduleID');
		for(var i=0;i<aSelectedStudents.length;i++){
			var oStudent = {};
			oStudent.studentID = aSelectedStudents[i].studentID;
			oStudent.totalHours = aSelectedStudents[i].totalHours;
			oStudent.contactHours = aSelectedStudents[i].contactHours;
			oStudent.cpeHours = aSelectedStudents[i].cpeHours;
			oStudent.creditHours = aSelectedStudents[i].creditHours;
			oStudent.grade = aSelectedStudents[i].grade;
			oStudent.completionStatus = aSelectedStudents[i].completionStatus;
			oStudent.gradingOption = aSelectedStudents[i].gradingOption;
			if(!oStudent.completionStatus && aSelectedStudents[i].completionStatuses){
				var aCompletionStatus = this.getCreditedCompletionStatuses(aSelectedStudents[i].completionStatuses);
				oStudent.completionStatus = aCompletionStatus[0].cmplStatusID
			}
			if(!oStudent.grade && aSelectedStudents[i].gradeValues){
				oStudent.grade = aSelectedStudents[i].gradeValues[0].valueID
			}
			oStudent.comment = (aSelectedStudents[i].comment)?aSelectedStudents[i].comment:'';
			aStudents.push(oStudent);
		}
		jQuery.ajax(this.getComponent().getService().recordLearning(sScheduleID,aStudents)).done(function(oResponse) {
			var oViewPropertiesModel = self.getComponent().getModel('viewPropertiesModel');
			oViewPropertiesModel.setProperty('/reloadStudentsTab', true);
			self.getEventBus().publish('updateRecordedStudent',{recordedStudentCount:aStudents.length});
			self.getView().setBusy(false);
			self.getRouter().myNavBack("SO_DETAILS", {
				scheduleID: sScheduleID
			});
		}).fail(function(){
			self.getView().setBusy(false);
		});
	},
	getCreditedCompletionStatuses : function(completionStatuses){
		return completionStatuses.filter(function (oCompletionStatus) {
			return (oCompletionStatus.provideCredit === true);
		});
	},
	isConfirmButtonEnabled : function(iLength) {
		return (iLength>0);
	},
	isCompletionStatusVisible : function(gradingOption){
		return (gradingOption == 0);
	},
	isGradeInputFieldVisible : function(gradingOption){
		return (gradingOption !== 2);
	},
	isGradeOptionFieldVisible : function(gradingOption){
		return (gradingOption === 2);
	},
	isGradeRequired : function(gradingOption){
		return (gradingOption === 1 || gradingOption === 2);
	},
	gradeValue : function(valueID,valueDescription){
		if(valueDescription==null) return valueID;
		return valueDescription;
	},
	getCompletionStatusSeletedKey:function(completionStatuses){
		if(completionStatuses){
			var selectableStatus = this.getCreditedCompletionStatuses(completionStatuses)
			return selectableStatus[0].cmplStatusID;
		}
	},
	isUserPhotoVisble:function(isUserPhotosHideGlobally){
		return (isUserPhotosHideGlobally == false);
	},
	
	getRecordLearningCommentIcon:function(sComment){
		if(!sComment || sComment === ""){
			return 'sap-icon://comment';
		}else{
			return 'sap-icon://notes';
		}
	},

	getStudentImageUrl : function(imageUrl) {
		var appParamsModelData = this.getComponent().getModel('appParamsModel').getData();
		return (imageUrl ? imageUrl : appParamsModelData.contextPath + "/ui/common/img/defaultIcons/photoNotAvailable.gif");
	},
	getStudentName:function(studentFName, studentMI, studentLName,studentId) {
		if(!studentFName && !studentMI && !studentLName){
			return studentId;
		}
		return this.getComponent().getModel("i18nModel").getLabel("format.FirstName.MI.LastName", [(studentFName ? studentFName : ''), (studentMI ? studentMI : ''), (studentLName ? studentLName : '')]);
	},
	getStudSegAttendanceText:function(studSegAttendanceCount, schedSegmentsCount) {
		return this.getComponent().getModel("i18nModel").getLabel("label.scheduleInstance.segmentAttended.segmentCount", [studSegAttendanceCount, schedSegmentsCount] );
	},
	getCompletionStatus:function(completionStatusId,CompletionStatusDesc){
		if(CompletionStatusDesc) {
			return CompletionStatusDesc;
		}else{
			return completionStatusId;
		}
	},
	setScheduleOfferingSelectedTab:function(selectedTab){
		var oViewPropertiesModel = this.getComponent().getModel('viewPropertiesModel');
		oViewPropertiesModel.setProperty('/scheduleOfferingSelectedTab',selectedTab);
	},
		
	getComponent: function () {
		return sap.ui.component(sap.ui.core.Component.getOwnerIdFor(this.getView()));
	},
	
	getRouter: function () {
		return sap.ui.core.UIComponent.getRouterFor(this);
	},
	getEventBus : function() {
		return sap.ui.getCore().getEventBus();
	}
});

