sap.ui.controller("sap.sf.learning.instructor.view.ScheduleOfferingDetails", {

	onInit : function() {		
		this._studentsCahce = {};
		this._accessSendEvaluationsCache = {};
		this._accessRecordLearningCache = {};
		this._recordLearningDefaultDataCahce = {};
		this.getRouter().attachRouteMatched(this.onRouteMatched, this);
		this._summaryCahce = {};
		var i18nModel = this.getComponent().getModel("i18nModel");
		
		this.getView().byId('segmentOverflowToolbar').addEventDelegate({
			onAfterRendering : function(oEvent) {
				oEvent.srcControl.$().attr('role','presentation');
				$.each($('.scheduleOfferingDetailsRow'), function (index, value) {
					var $row = $(value);
					var segmentTitle = $row.find('.sapMObjectIdentifierTitle').text();
					var segmentType = $row.find('.segmentStatusIcon').text();
					if(segmentType == "Past Segment") {
						$row.find('.attendanceButton').attr('aria-label',i18nModel.getLabel('label.EditAttendanceSegmentTitle', [segmentTitle]));
					}
					else {
						$row.find('.attendanceButton').attr('aria-label',i18nModel.getLabel('label.RecordAttendanceSegmentTitle', [segmentTitle]));
					}		
					$row.find('.startVlsButton').attr('aria-label',i18nModel.getLabel('label.StartVlsSegmentTitle', [segmentTitle]));
				});		
			}
		});
		
		this.getView().byId('scheduleOfferingTable').addEventDelegate({			
			onAfterRendering : function(oEvent) { 
				oEvent.srcControl.$().find('table tr:first-child').removeClass('sapMListTblHeaderNone'); //Removing Default SAPUI5 Class to display first row with border					
			}
		});
	},
	
	onNavBack: function (oEvent) {			
		this.getRouter().myNavBack("HOME", {
			scheduleStatus: this._scheduleStatus 
		});
	},

	onRouteMatched : function(oEvent) {
		var sRoute = oEvent.getParameter("name");
		var oParams = oEvent.getParameter("arguments");
		if (sRoute !== "SO_DETAILS")
			return;
		
		//As soon as the page loads, we have to set some defaults until the page finishes loading.
		this.getView().setModel(new sap.sf.learning.ui5.model.DataModel());
		this.getView().getModel().setProperty('/hasPermission', false);	
		
		this._scheduleID = oParams.scheduleID;
		this._scheduleStatus = oParams.scheduleStatus;
		this.getComponent().getModel('viewPropertiesModel').setProperty('/cpntTitle', decodeURIComponent(oParams.itemTitle));
		this.getView().byId('idIconTabBar').setSelectedKey(this.getScheduleOfferingSelectedTab());
		var oViewPropertiesModel = this.getComponent().getModel('viewPropertiesModel');
		var reloadStudentsTab = oViewPropertiesModel.getProperty('/reloadStudentsTab');
		var self = this;
		this.getView().setBusy(true);
		var oServices = this.getComponent().getService();
		var isFutureOffering = this.getScheduleOfferingsMode()==='FUTURE'? true: false;
				
		var oScheduleSegmentsTabServices = [oServices.getScheduleSegments(this._scheduleID)];
		if(!isFutureOffering) {
			oScheduleSegmentsTabServices.push(oServices.hasAccessToSendEvaluations(this._scheduleID, this.getScheduleOfferingsMode()));
		}
		jQuery.when.apply(this, jQuery.map(oScheduleSegmentsTabServices, jQuery.ajax)).done(function() {
			var oScheduleSegmentsData = Array.isArray(arguments) ? arguments[0] : arguments;
			var segments = sap.sf.learning.ui5.model.DataModel.getDataFromRestOperationStatusVOX(oScheduleSegmentsData[0]);
			var oModel = self.getView().getModel();
			oModel.setData(segments);
			if(!isFutureOffering) {
				var oEvaluations = arguments[1];
    			var bSendEvaluationAccessMap = sap.sf.learning.ui5.model.DataModel.getDataFromRestOperationStatusVOX(oEvaluations[0]);
    			self.getView().getModel().setProperty('/hasPermission', bSendEvaluationAccessMap.hasPermission);
    			self.getView().getModel().setProperty('/isEnabled', bSendEvaluationAccessMap.isEnabled);
    		}else {
    			self.getView().getModel().setProperty('/hasPermission', false);
    			self.getView().getModel().setProperty('/isEnabled', false);
			}
			if(reloadStudentsTab) {
				var oModel = self.getView().getModel("studentsModel");
				self._onLoadStudentsData(oModel);
			}else {
				self.getView().setBusy(false);
			}
		});	
		var pageTitleString = this.getComponent().getModel("i18nModel").getLabel('label.CourseDetails');
		document.title = pageTitleString;
	},
	
	_onLoadStudentsData : function(oModel){
		var self = this;
		var sScheduleID = this._scheduleID;		
		var isFutureOffering = this.getScheduleOfferingsMode()==='FUTURE'? true: false;
		var oServices = this.getComponent().getService();
		var oViewPropsModel = self.getComponent().getModel('viewPropertiesModel')
		
		var oStudentsTabServices = [oServices.getStudents(sScheduleID,this.getScheduleOfferingsMode())];
		if(!isFutureOffering) {
			oStudentsTabServices.push(oServices.hasAccessToRecordLearning(sScheduleID,this.getScheduleOfferingsMode()));
		}
		 jQuery.when.apply(this, jQuery.map(oStudentsTabServices, jQuery.ajax)).done(function() {
			var oStudentsData = Array.isArray(arguments) ? arguments[0] : arguments;
			var oStudents = sap.sf.learning.ui5.model.DataModel.getDataFromRestOperationStatusVOX(oStudentsData[0]);
			oModel.setData(oStudents);
			self._studentsCahce[sScheduleID] = oStudents;
			oViewPropsModel.setProperty('/reloadStudentsTab',false);
			if(!isFutureOffering) {
    			var oRecordLearning = arguments[1];
    			var bAccessRecordLearning = sap.sf.learning.ui5.model.DataModel.getDataFromRestOperationStatusVOX(oRecordLearning[0]);
    			self._accessRecordLearningCache[sScheduleID] = bAccessRecordLearning;
    			self.getView().getModel().setProperty('/hasAccessToRecordLearning',bAccessRecordLearning);
			} else {
    			self._accessRecordLearningCache[sScheduleID] = false;
    			self.getView().getModel().setProperty('/hasAccessToRecordLearning',false);
			}
			self.getView().setBusy(false);
		});
		
	},
	
	_loadSummaryData : function(oModel){
		var self = this;
		var sScheduleID = this._scheduleID;
		var oServices = this.getComponent().getService();
		var oSummaryTabServices = [oServices.getScheduleOfferingSummary(sScheduleID)];
		 jQuery.when.apply(this, jQuery.map(oSummaryTabServices, jQuery.ajax)).done(function() {
			var oSummaryData = Array.isArray(arguments) ? arguments[0] : arguments;
			var oSummary = sap.sf.learning.ui5.model.DataModel.getDataFromRestOperationStatusVOX(oSummaryData[0]);
			oModel.setData(oSummary);
			self._summaryCahce[sScheduleID] = oSummary;
			self.getView().setBusy(false);
		}); 
	},

	getCourseSizeText : function(minRegistration, maxRegistration) {
		var i18nModel = this.getComponent().getModel('i18nModel');
		if (!minRegistration && !maxRegistration) {
			return i18nModel.getLabel('label.NoLimit');
		} else if (!minRegistration) {
			return maxRegistration;
		} else if (!maxRegistration) {
			return minRegistration + " - " + i18nModel.getLabel('label.NoLimit');
		} else {
			return minRegistration + " - " + maxRegistration;
		}
	},
	
	getCourseDatesText : function(startDate, endDate) {
		return startDate + " - " + endDate;
	},
	
	getSegmentStatusIcon : function(segmentPeriodStatus) {
		switch (segmentPeriodStatus) {
			case 'PAST':
				return 'sap-icon://accept';
		}
		return '';
	},
	
	getSegmentStatusIconColor : function(segmentPeriodStatus) {
		return (segmentPeriodStatus == 'FUTURE' ? 'Default' : '#007833');
	},
		
	getSegmentStatusIconToolTip : function(segmentPeriodStatus) {
		var i18nModel = this.getComponent().getModel("i18nModel");
		return (segmentPeriodStatus == 'PAST' ? i18nModel.getLabel('label.PastSegment') : '');
	},
	
	getSegmentStatusIconLabel : function(segmentPeriodStatus) {
		var i18nModel = this.getComponent().getModel("i18nModel");
		return (segmentPeriodStatus == 'PAST' ? i18nModel.getLabel('label.PastSegment') : '');
	},
		
	getFacility : function(facilityAddress, facilityCity, facilityState, facilityZipCode, facilityCountry) {
		var sAddress = ((facilityAddress ?  facilityAddress : '') + (facilityCity ? ', ' + facilityCity : '') +  (facilityState ? ', ' + facilityState : '') + (facilityZipCode ? ', ' + facilityZipCode : '') + (facilityCountry ? ', ' + facilityCountry : ''));
		return sAddress ? sAddress : this.getComponent().getModel("i18nModel").getLabel('label.NoFacility');
	},

	getIsVleEnabled : function(isVleEnabled, joinVirtualSession) {
		return (isVleEnabled === true && joinVirtualSession === true);
	},

	launchVlsSession : function(event) {
		var segmentVO = event.getSource().getBindingContext().getObject();
		if (segmentVO && segmentVO.launchVlsURL) {
			var bNotAWebApp = !sap.sf.learning.instructor.util.Formatter.isWebApp();
			if(bNotAWebApp) {
				var url = "ios://launchVLS?url=" + encodeURIComponent(segmentVO.launchVlsURL);
				window.location.href = url;
			} else {
				sap.m.URLHelper.redirect(segmentVO.launchVlsURL, true);
			}
		}
	},

	iconTabBarChanged : function(oEvent) {
		//setting it to null so that it won't show previous model/data
		this.getView().setModel(null,"studentsModel");
		this.getView().setModel(null,"summaryModel");
		
		var sSelectedKey = oEvent.getParameter("selectedKey");
		var sScheduleID = this._scheduleID;
		var self = this;
		var oModel;
		this.setScheduleOfferingSelectedTab(sSelectedKey);
		switch (sSelectedKey) {
			case "STUDENTS":
				
				oModel = new sap.sf.learning.ui5.model.DataModel();
				this.getView().setModel(oModel, "studentsModel");
				
				if (this._studentsCahce.hasOwnProperty(sScheduleID)) {
					oModel.setData(this._studentsCahce[sScheduleID]);
					this.getView().getModel().setProperty('/hasAccessToRecordLearning',this._accessRecordLearningCache[sScheduleID]);
					return;
				}
				this.getView().setBusy(true);
				this._onLoadStudentsData(oModel);
				break;
			case "ABOUT":
				oModel = new sap.sf.learning.ui5.model.DataModel();
				this.getView().setModel(oModel, "summaryModel");

				if (this._summaryCahce.hasOwnProperty(sScheduleID)) {
					oModel.setData(this._summaryCahce[sScheduleID]);
					return;
				}
				this.getView().setBusy(true);
				this._loadSummaryData(oModel);
				break;
		}
	},

	_fetchRecordLearningDefaultData : function(sScheduleID) {
		if (this._recordLearningDefaultDataCahce.hasOwnProperty(sScheduleID)) {
			this.getView().setBusy(false);
			this.fireEvent('buildRecordLearningModel');
		} else {
			var self = this;
			jQuery.ajax(this.getComponent().getService().getRecordLearningData(sScheduleID)).done(function(oResponse) {
				var oRecordLearningData = sap.sf.learning.ui5.model.DataModel.getDataFromRestOperationStatusVOX(oResponse);
				self._recordLearningDefaultDataCahce[sScheduleID] = oRecordLearningData;
				self.getView().setBusy(false);
				self.fireEvent('buildRecordLearningModel');
			});
		}
	},

	onNavToRecordAttendance : function(oEvent) {
		 var oSegmentVO = oEvent.getSource().getBindingContext().getObject();
		 var scheduleID = oSegmentVO.scheduleID;
		 var segmentNumber =oSegmentVO.scheduleSegmentNo;
		 var segmentDesc = oSegmentVO.segmentDesc;
 		 var qrScanner = !sap.sf.learning.instructor.util.Formatter.isWebApp();
		 var self = this;
		if(qrScanner) {
			var url = "ios://openQRscanner?scheduleID=" + scheduleID + "&segmentNo=" + segmentNumber+"&selectedTab=" + ( oSegmentVO.segmentPeriodStatus === 'PAST' ? 'MANUAL' : 'SCAN' );
			window.location.href = url;
			window.iosQRScannerExitCallback = function(qrCode) {
				self.hideRecordAttendanceDialog();
			};
			
			window.openSegRecordAttendanceDialog = function(schdID, segNo) {
				self.openRecordAttendanceDialog(schdID, segNo, segmentDesc);
			};
		}else {
			this.openRecordAttendanceDialog(scheduleID, segmentNumber, segmentDesc);
		}
		
	},
	
	openRecordAttendanceDialog : function(scheduleID, scheduleSegmentNo, segmentDesc) {
		var self = this;

		jQuery.ajax(self.getComponent().getService().getScheduleSegmentStudentsForAttendance(scheduleID, scheduleSegmentNo)).done(function(oResponse) {
			var oSegAttendanceStudents = sap.sf.learning.ui5.model.DataModel.getDataFromRestOperationStatusVOX(oResponse);
			var oSegAttendanceModel = new sap.sf.learning.ui5.model.DataModel({
				studAttendanceList : oSegAttendanceStudents,
				isCheckAllSelected : false,
				studentsAttendedCount : 0,
				studTotalAttendanceCount : 0
			});
			self.getView().setModel(oSegAttendanceModel, "segAttendanceModel");

			self._setSegAttendanceAdditionalData();

			var i18nModel = self.getComponent().getModel("i18nModel");
			var isMobileApp = !sap.sf.learning.instructor.util.Formatter.isWebApp();

			if (!self._recAttendanceDialog) {
				self._recAttendanceDialog = new sap.m.Dialog({
					stretch : isMobileApp,
					afterOpen : function(oEvent) {
						//this will set focus to entire dialog so that 508 will work properly and JAWS will read entire Dialog rather than focusing on single element.
						oEvent.getSource().focus();
					}
				});
				self._recAttendanceDialog.addContent(sap.ui.xmlfragment("sap.sf.learning.instructor.fragment.SegmentRecordAttendance", self));

				if (!isMobileApp) {
					var oDoneButton = new sap.m.Button({
						text : i18nModel.getLabel("button.Done"),
						press : function() {
							self.hideRecordAttendanceDialog();
						},
						type : sap.m.ButtonType.Emphasized
					})
					self._recAttendanceDialog.addButton(oDoneButton);
				}
				
				var subHeaderBar = new sap.m.Bar({
					contentLeft : [new sap.m.CheckBox({
						text : {
							parts : ["segAttendanceModel>/studentsAttendedCount", "segAttendanceModel>/studTotalAttendanceCount"],
							formatter : function(studentsAttendedCount, studTotalAttendanceCount) {
								return (studentsAttendedCount === studTotalAttendanceCount) ? i18nModel.getLabel("label.RemoveAttendanceForAll") : i18nModel.getLabel("label.MarkAllAttended");
							}
						},
						tooltip : {
							parts : ["segAttendanceModel>/studentsAttendedCount", "segAttendanceModel>/studTotalAttendanceCount"],
							formatter : function(studentsAttendedCount, studTotalAttendanceCount) {
								return (studentsAttendedCount === studTotalAttendanceCount) ? i18nModel.getLabel("label.RemoveAttendanceForAll") : i18nModel.getLabel("label.MarkAllAttended");
							}
						},
						visible : {
							parts : ["segAttendanceModel>/studAttendanceList/length", "segAttendanceModel>/isRecordLearningCompletedForAll"],
							formatter : function(segStudsCount, isRecordLearningCompletedForAll) {
								return ( ( !!(segStudsCount && segStudsCount !== 0) ) && !isRecordLearningCompletedForAll );
							}
						},
						select : function(oEvent) {
							self.showCheckAllRecordAttendanceConfirmation(oEvent);
						},
						selected : "{segAttendanceModel>/isCheckAllSelected}"
					})],
					contentMiddle : [new sap.m.Text({
						text : i18nModel.getLabel("label.Instructor.NoRegisteredStudentsMessage"),
						tooltip : i18nModel.getLabel("label.Instructor.NoRegisteredStudentsMessage"),
						visible : {
							path : "segAttendanceModel>/studAttendanceList/length",
							formatter : function(segStudsCount) {
								return (!segStudsCount);
							}
						}
					})],
					contentRight : [new sap.m.Text({
						text : {
							path : "segAttendanceModel>/studentsAttendedCount" ,
							formatter : function(studentsAttendedCount) {
								return i18nModel.getLabel("label.StudentsAttendedCount", studentsAttendedCount );
							}
						},
						tooltip : {
							path : "segAttendanceModel>/studentsAttendedCount" ,
							formatter : function(studentsAttendedCount) {
								return i18nModel.getLabel("label.StudentsAttendedCount", studentsAttendedCount );
							}
						},
						visible : {
							path : "segAttendanceModel>/studAttendanceList/length",
							formatter : function(segStudsCount) {
								return !!( segStudsCount && segStudsCount !== 0 );
							}
						}
					})]
				});
				self._recAttendanceDialog.setSubHeader(subHeaderBar);
				self.getView().addDependent(self._recAttendanceDialog);
			}
			
			self._recAttendanceDialog.setTitle(isMobileApp ? i18nModel.getLabel("label.RecordAttendance.SegmentDesc", segmentDesc) : i18nModel.getLabel("label.RecordAttendance.Title", segmentDesc));
			self._recAttendanceDialog.open();
		});
	},
	
	_setSegAttendanceAdditionalData : function(){
		var oSegAttendanceModel = this.getView().getModel('segAttendanceModel');
		var oSegAttendanceModelData = oSegAttendanceModel.getData();
		var aStudentsAttenVoxs = oSegAttendanceModelData.studAttendanceList;
		if(!aStudentsAttenVoxs){
			return;
		}
		var studTotalCount = 0;
		var studentsAttendedCount = 0;
		var studRecordLearningAtteCnt = 0;
		var isRecordLearningCompletedForAll = true;
		for (var j = 0; j < aStudentsAttenVoxs.length; j++) {
			if(!aStudentsAttenVoxs[j].recordLearningCompleted){
				studTotalCount = studTotalCount +1;
				if(aStudentsAttenVoxs[j].attended){
					studentsAttendedCount = studentsAttendedCount+1;	
				}
				isRecordLearningCompletedForAll = false;
			}else if(aStudentsAttenVoxs[j].attended){
				studRecordLearningAtteCnt = studRecordLearningAtteCnt+1;
			}
		}
		var studTotalAttendanceCount = studTotalCount + studRecordLearningAtteCnt;
		studentsAttendedCount = studentsAttendedCount + studRecordLearningAtteCnt;
		
		oSegAttendanceModel.setProperty( "/studTotalAttendanceCount" , studTotalAttendanceCount );	
		oSegAttendanceModel.setProperty( "/studentsAttendedCount" , studentsAttendedCount );
		oSegAttendanceModel.setProperty( "/isCheckAllSelected" , studTotalAttendanceCount === studentsAttendedCount );
		oSegAttendanceModel.setProperty( "/isRecordLearningCompletedForAll" , isRecordLearningCompletedForAll );
	},
	
	hideRecordAttendanceDialog : function() {
		if(this.getView().getBusy()){
			this.getView().setBusy(false);
		}
		this._recAttendanceDialog.close();
	},
	
	handleStudCompleteRecordAttendance : function(oEvent){
		var oStudentVo = oEvent.getSource().getBindingContext("segAttendanceModel").getObject();
		var aSchedSegAttendanceVos = [{ scheduleID : oStudentVo.scheduleID , scheduleSegmentNo : oStudentVo.scheduleSegmentNo , attended : oStudentVo.attended , comment : oStudentVo.comment , studentID : oStudentVo.studentID }];
		this.submitSegmentRecordAttendance(oStudentVo.scheduleID, oStudentVo.scheduleSegmentNo, aSchedSegAttendanceVos);
		
		var oSegAttendanceModel = this.getView().getModel("segAttendanceModel");
		var studentsAttendedCount = oSegAttendanceModel.getData().studentsAttendedCount;
		studentsAttendedCount = oStudentVo.attended ? (studentsAttendedCount+1) : (studentsAttendedCount-1);
		oSegAttendanceModel.setProperty("/studentsAttendedCount" , studentsAttendedCount );
		oSegAttendanceModel.setProperty("/isCheckAllSelected" , studentsAttendedCount ===  oSegAttendanceModel.getData().studTotalAttendanceCount );
	},
	
	showCheckAllRecordAttendanceConfirmation : function(oEvent) {
		var oSegAttendanceModel = this.getView().getModel('segAttendanceModel');
    	var oSegAttendanceModelData = oSegAttendanceModel.getData();
    	var i18nModel = this.getComponent().getModel("i18nModel");
    	var self = this;
    	sap.m.MessageBox.show(i18nModel.getLabel(oSegAttendanceModelData.isCheckAllSelected ? "label.RecordAttendance.CheckAll.Confirmation" : "label.RecordAttendance.UncheckAll.Confirmation"), {
    		title : i18nModel.getLabel("label.Confirmation"),
    		actions : [sap.m.MessageBox.Action.YES, sap.m.MessageBox.Action.NO],
    		onClose : function(oAction) {
    			if (oAction === 'YES') {
    				self.handleCheckAllCompleteRecordAttendance();
    			}
    			if(oAction === 'NO'){
    				oSegAttendanceModel.setProperty("/isCheckAllSelected", !oSegAttendanceModelData.isCheckAllSelected);
    			}
    		}
    	});
	},	
	
	handleCheckAllCompleteRecordAttendance : function(oEvent){
		var self = this;
		var aSelectedStudAtteVos = [];
		var oSelectedStudAtteVo = null;
		var oSegAttendanceModel = this.getView().getModel('segAttendanceModel');
		var oSegAttendanceModelData = oSegAttendanceModel.getData();
		var aStudentsAttenVoxs = oSegAttendanceModelData.studAttendanceList;
		var scheduleID = null;
		var scheduleSegmentNo = null;
		
		for (var j = 0; j < aStudentsAttenVoxs.length; j++) {
			scheduleID = aStudentsAttenVoxs[j].scheduleID;
			scheduleSegmentNo = aStudentsAttenVoxs[j].scheduleSegmentNo;
			if(!aStudentsAttenVoxs[j].recordLearningCompleted && aStudentsAttenVoxs[j].attended !== oSegAttendanceModelData.isCheckAllSelected){
				oSegAttendanceModel.setProperty("/studAttendanceList/"+j+"/attended",oSegAttendanceModelData.isCheckAllSelected);
				oSelectedStudAtteVo = {scheduleID : aStudentsAttenVoxs[j].scheduleID, scheduleSegmentNo : aStudentsAttenVoxs[j].scheduleSegmentNo, attended : aStudentsAttenVoxs[j].attended, comment : aStudentsAttenVoxs[j].comment, studentID : aStudentsAttenVoxs[j].studentID };
				aSelectedStudAtteVos.push(oSelectedStudAtteVo);
			}
		}
		
		if(aSelectedStudAtteVos && aSelectedStudAtteVos.length > 0){
			this.submitSegmentRecordAttendance(scheduleID, scheduleSegmentNo, aSelectedStudAtteVos);
		}
		
		var studTotalAttendanceCount = this.getView().getModel("segAttendanceModel").getProperty("/studTotalAttendanceCount");
		this.getView().getModel("segAttendanceModel").setProperty("/studentsAttendedCount" , oSegAttendanceModelData.isCheckAllSelected ? studTotalAttendanceCount : 0 );
	},
	
	submitSegmentRecordAttendance : function(scheduleID, scheduleSegmentNo, aStudSegAttendanceVos){
		var self = this;
		this.getView().setBusy(true);
		  jQuery.ajax(this.getComponent().getService().recordSegmentAttendance(scheduleID, scheduleSegmentNo,aStudSegAttendanceVos)).done(function(oResponse) {
			  self.getView().setBusy(false);
		  });
	},

	onNavToRecordLearning : function(oEvent) {
		var oI18nModel = this.getComponent().getModel("i18nModel");
		var oStudentsData = this.getView().getModel('studentsModel').getData();
		var aSelectedStudents = [];
		var aSelectedStudentIds = [];
		var bIsRecordedStudentSelected = false;
		for (var i = 0; i < oStudentsData.length; i++) {
			var aScheduleStudents = oStudentsData[i].scheduleStudents;
			for (var j = 0; j < aScheduleStudents.length; j++) {
				if (aScheduleStudents[j].isSelected) {
					aSelectedStudents.push(aScheduleStudents[j]);
					aSelectedStudentIds.push(aScheduleStudents[j].studentID);
					if (!bIsRecordedStudentSelected && (aScheduleStudents[j].studentSOStatus == 'LR')) {
						bIsRecordedStudentSelected = true;
					}
				}
			}
		}
		
		if (aSelectedStudents.length > 0 && !bIsRecordedStudentSelected) {
			this.getView().setBusy(true);
			var oServices = this.getComponent().getService();
			var sScheduleID = this._scheduleID;
			var loadSchedDefaultApi = !this._recordLearningDefaultDataCahce.hasOwnProperty(sScheduleID);
			var oStudentsServices = [oServices.getSchedSegsStudentsAttendanceCount(sScheduleID,aSelectedStudentIds)];
			if(loadSchedDefaultApi) {
				oStudentsServices.push(oServices.getRecordLearningData(sScheduleID));
			}
			var self = this;
			var aStudSegsAttendanceCount=null;
			
			jQuery.when.apply(this, jQuery.map(oStudentsServices, jQuery.ajax)).done(function() {
				var oRestDataResults = Array.isArray(arguments) ? arguments[0] : arguments;
				aStudSegsAttendanceCount = sap.sf.learning.ui5.model.DataModel.getDataFromRestOperationStatusVOX(oRestDataResults[0]);
				
				if(loadSchedDefaultApi) {
	    			var oRecordLearningDefaultData = sap.sf.learning.ui5.model.DataModel.getDataFromRestOperationStatusVOX(oRestDataResults[1]);
	    			var aSchedSegments = self.getView().getModel().getProperty("/segments");
	    			oRecordLearningDefaultData.schedSegmentsCount = aSchedSegments ? aSchedSegments.length : 0; 
	    			self._recordLearningDefaultDataCahce[sScheduleID] = oRecordLearningDefaultData;
	    		} 
				self.getView().setBusy(false);
				
				var aRecordLearningStudents =[]; 
				var oRecordLearningData = self._recordLearningDefaultDataCahce[sScheduleID]
				for (var i = 0; i < aSelectedStudents.length; i++) {
					var studSegAttendanceCount = aStudSegsAttendanceCount[aSelectedStudents[i].studentID];
					var mergedStudent = jQuery.extend(aSelectedStudents[i], oRecordLearningData,{studSegAttendanceCount : (studSegAttendanceCount ? studSegAttendanceCount : 0)});
					aRecordLearningStudents.push(mergedStudent);
				}
				
				var oStudent = jQuery.extend(aScheduleStudents[j], oRecordLearningData);
				var oModel = self.getView().getModel();
				
				var oRecordLearningData = {
					scheduleID : sScheduleID,
					selectedStudents : aRecordLearningStudents
				};
				
				var oRecordLearningModel = self.getComponent().getModel('recordLearningModel');
				if (!oRecordLearningModel) {
					oRecordLearningModel = new sap.sf.learning.ui5.model.DataModel(oRecordLearningData);
					self.getComponent().setModel(oRecordLearningModel, 'recordLearningModel');
				}
				oRecordLearningModel.setData(oRecordLearningData);
				
				self.getRouter().navTo("RECORD_LEARNING", {
					scheduleID : sScheduleID
				});
			});
		} else {
			
			this.getComponent().getModel("i18nModel").getLabel("format.FirstName.MI.LastName");
			
			var bCompact = !!this.getView().$().closest(".sapUiSizeCompact").length;
			var sErrorMsg = oI18nModel.getLabel("error.RecordLearning.Student.Required");
			if (bIsRecordedStudentSelected) {
				sErrorMsg = oI18nModel.getLabel("error.RecordLearning.Student.AlreadyRecorded");
			}
			sap.m.MessageBox.alert(sErrorMsg, {
				styleClass : bCompact ? "sapUiSizeCompact" : ""
			});
		}
	},
	
	selectStudent : function(oEvent) {
		var oModel = this.getView().getModel("studentsModel");
		var oCheckbox = oEvent.getSource();
		var sPath = oCheckbox.getBindingContext("studentsModel").getPath();
		var aStudents = oModel.getProperty(sPath + '/scheduleStudents');
		oModel.setProperty(sPath + '/isSelected', oCheckbox.getSelected());
		for (var i = 0; i < aStudents.length; i++) {
			oModel.setProperty(sPath + '/scheduleStudents/' + i + '/isSelected', oCheckbox.getSelected());
		}
	},

	navigateToSegmentDetails : function(oEvent) {
		var oSegmentVO = oEvent.getSource().getBindingContext().getObject();
		this.getRouter().navTo("SEGMENT_DETAILS", {
			scheduleID : oSegmentVO.scheduleID,
			segmentID : oSegmentVO.scheduleSegmentNo
		});

		var viewPropertiesModel = this.getComponent().getModel("viewPropertiesModel");
		viewPropertiesModel.setProperty("/segmentDescFromSO",oSegmentVO.segmentDesc);
	},

	isUserPhotoVisble : function(isUserPhotosHideGlobally) {
		return (isUserPhotosHideGlobally == false);
	},

	getStudentStatusStateIcon : function(isPrerequisitesSatisfied) {
		return (isPrerequisitesSatisfied == false);
	},

	getStudentStatusState : function(isPrerequisitesSatisfied) {
		return (isPrerequisitesSatisfied === true ? 'None' : 'Error');
	},

	getStudentName : function(studentFName, studentMI, studentLName, studentId) {
		if (!studentFName && !studentMI && !studentLName) {
			return studentId;
		}
		return this.getComponent().getModel("i18nModel").getLabel("format.FirstName.MI.LastName",
																  [(studentFName ? studentFName : ''), (studentMI ? studentMI : ''), (studentLName ? studentLName : '')]);
	},
	isFailStatusVisible : function(bProvideCredit) {
		return (bProvideCredit == false);
	},
	isStudentSOStatusVisible : function(studentSOStatus) {
		return studentSOStatus == 'LR';
	},

	getStudentSOStatusState : function(bProvideCredit) {
		return (bProvideCredit === true ? 'Success' : 'Error');
	},

	getCompletionStatus : function(completionStatusId, CompletionStatusDesc) {
		if (CompletionStatusDesc) {
			return CompletionStatusDesc;
		} else {
			return completionStatusId;
		}
	},

	isPanelVisible : function(iCount) {
		return (iCount > 0);
	},

	isNoDataTextVisible : function(iCount) {
		return (iCount < 1);
	},

	isFooterVisible : function(sSelectedTab) {
		return (sSelectedTab == 'STUDENTS');
	},

	isToolbarVisible : function(iGroupCount) {
		return (iGroupCount > 0);
	},

	setScheduleOfferingSelectedTab : function(selectedTab) {
		var oViewPropertiesModel = this.getComponent().getModel('viewPropertiesModel');
		oViewPropertiesModel.setProperty('/scheduleOfferingSelectedTab', selectedTab);
	},

	getStudentImageUrl : function(imageUrl) {
		var appParamsModelData = this.getComponent().getModel('appParamsModel').getData();
		return (imageUrl ? imageUrl : appParamsModelData.contextPath + "/ui/common/img/defaultIcons/photoNotAvailable.gif");
	},

	getScheduleOfferingStudentStatus : function(sStatus, iCount) {
		var i18nModel = this.getComponent().getModel('i18nModel');
		var sScheduleOfferingsMode = this.getScheduleOfferingsMode();
		var sLabelKey = "";
		switch (sStatus) {
			case "E":
				if (sScheduleOfferingsMode === 'FUTURE') {
					sLabelKey = "label.RegisteredStudents";
				} else {
					sLabelKey = "label.StudentsToRecord";
				}
				break;
			case "W":
				sLabelKey = "label.WaitingListStudents";
				break;
			case "LR":
				sLabelKey = "label.RecordLearningFinished";
				break;
		}
		return i18nModel.getLabel(sLabelKey, [iCount]);
	},

	sendEmail : function() {
		var oModel = this.getView().getModel("studentsModel");
		var i18nModel = this.getComponent().getModel('i18nModel');

		var aManageStudents = oModel.getData();
		var aStudents, oStudent;
		var aNoEmailStudents = [];

		this._sEmailListWithDelimiter = '';
		for (var i = 0, len = aManageStudents.length; i < len; i++) {
			aStudents = aManageStudents[i].scheduleStudents;
			for (var j = 0, lenj = aStudents.length; j < lenj; j++) {
				oStudent = aStudents[j];
				if (oStudent.isSelected === true) {
					if (oStudent.email) {
						this._sEmailListWithDelimiter += oStudent.email + ';';
					} else {
						aNoEmailStudents.push(oStudent);
					}
				}
			}
		}

		if (!this._sEmailListWithDelimiter && aNoEmailStudents.length == 0) {
			sap.m.MessageBox.alert(i18nModel.getLabel('label.instructor.SelectStudsToSendEmail'));
			return;
		}

		if (aNoEmailStudents.length > 0) {
			if (!this._sendEmailDialog) {
				this._sendEmailDialog = sap.ui.xmlfragment("sap.sf.learning.instructor.fragment.SendEmailDialog", this);
				this.getView().setModel(new sap.sf.learning.ui5.model.DataModel(aNoEmailStudents), "noEmailStudentsModel");
				this.getView().addDependent(this._sendEmailDialog);
			} else {
				this.getView().getModel("noEmailStudentsModel").setData(aNoEmailStudents);
			}
			this._sendEmailDialog.open();
		} else {
			this._triggerEmailClient(this._sEmailListWithDelimiter);
		}
	},

	_triggerEmailClient : function(sEmailList) {
		if (sEmailList) {
			var cpntTitle = this.getComponent().getModel('viewPropertiesModel').getProperty('/cpntTitle');
			sap.m.URLHelper.triggerEmail(sEmailList, cpntTitle);
		}
	},

	onSendEmailDialogCancel : function() {
		this._sendEmailDialog.close();
	},
	
	onSendEmailDialogYes : function() {
		this._triggerEmailClient(this._sEmailListWithDelimiter);
		this._sendEmailDialog.close();
	},

	getScheduleOfferingsMode : function() {
		return this.getComponent().getModel('viewPropertiesModel').getProperty('/scheduleOfferingsMode');
	},

	getScheduleOfferingSelectedTab : function() {
		return this.getComponent().getModel('viewPropertiesModel').getProperty('/scheduleOfferingSelectedTab');
	},
	
	sendEvaluations : function() {
		var sScheduleID = this.getComponent().getModel('viewPropertiesModel').getProperty('/scheduleID');
		var that = this;
		that.getView().setBusy(true);
		jQuery.ajax(that.getComponent().getService().sendEvaluations(sScheduleID)).done(function(oResponse) {
			var sSuccessMsg = that.getComponent().getModel("i18nModel").getLabel("label.successfullySentEvaluations");
			that.getView().getModel().setProperty('/isEnabled',false);
			that._accessSendEvaluationsCache[sScheduleID] = false;
			sap.m.MessageBox.alert(sSuccessMsg, {
				styleClass : ""
			});
		});	
		this.getView().setBusy(false);
	},
	
	getIsSendEvaluationEnabled: function (hasAccess) {
		var isEnabled = this.getView().getModel().getProperty('/isEnabled');
		return isEnabled;
	},
	
	getHasAccessToSendEvaluations : function (hasAccess, sScheduleID) {
		return hasAccess;
	},
	getHasAccessToRecordLearning : function (hasAccess, sScheduleID) {
		if(hasAccess==undefined) {
    		var	bHasAccess = this._accessRecordLearningCache[sScheduleID];
    		if(bHasAccess === undefined) {
    			return false;
    		}else {
    			return !!bHasAccess;
    		}
    	}
		return hasAccess;
	},
		
	getSegmentStatusIconColumnWidth : function(browserName){
		if(browserName === 'cr' || browserName === 'sf' || ((browserName === 'ie' && sap.ui.Device.browser.version < 10))){
			return "6%";
		}
		return "2rem";
	},
	
	getSegmentTimingColumnWidth : function(browserName){
		if(browserName === 'cr' || browserName === 'sf' || (browserName === 'ie' && sap.ui.Device.browser.version < 10)){
			return "10%";
		}
		return "6rem";
	},
			
	getButtonType: function(segmentPeriodStatus) {
		return segmentPeriodStatus == 'CURRENT' ? "Emphasized" : "Default" ;
	},
			
	getComponent : function() {
		return sap.ui.component(sap.ui.core.Component.getOwnerIdFor(this.getView()));
	},

	getRouter : function() {
		return sap.ui.core.UIComponent.getRouterFor(this);
	},

	getEventBus : function() {
		return sap.ui.getCore().getEventBus();
	}
});
