sap.ui.controller("sap.sf.learning.instructor.view.SegmentDetails", {
	
	onInit: function() {
		this._segmentDetailsCahce = {};
		this.getRouter().attachRouteMatched(this.onRouteMatched, this);
	},
	
	onRouteMatched: function (oEvent) {
		var sRoute = oEvent.getParameter("name");
		var oParams = oEvent.getParameter("arguments");
		
		if(sRoute !== "SEGMENT_DETAILS") return;
		
		this._scheduleID = oParams.scheduleID;
		var segmentKey = this._getSegmentKey(oParams.scheduleID, oParams.segmentID);
		if(this._segmentDetailsCahce.hasOwnProperty(segmentKey)) {
			this.getView().getModel().setData(this._segmentDetailsCahce[segmentKey]);
			return;
		}
		
		var self = this;
		this.getView().setBusy(true);
		jQuery.ajax(this.getComponent().getService().getSegmentDetails(oParams.scheduleID, oParams.segmentID)).done(function(oResponse) {
			var segmentDetails = sap.sf.learning.ui5.model.DataModel.getDataFromRestOperationStatusVOX(oResponse);
			var oModel = self.getView().getModel(); 
			if(!oModel) {
				oModel = new sap.sf.learning.ui5.model.DataModel(segmentDetails);
				self.getView().setModel(oModel);
			}
			oModel.setData(segmentDetails);
			self._segmentDetailsCahce[segmentKey] = segmentDetails;
			self.getView().setBusy(false);
		});
		var pageTitleString = this.getComponent().getModel("i18nModel").getLabel('label.instructor.SegmentDetails');
		document.title = pageTitleString;
	},
	
	getLocationDesc : function(locationDesc, locationId, isPrimary){
		var i18nModel = this.getComponent().getModel('i18nModel');
		var formattedDesc = this.getFormattedDesc(locationDesc, locationId);
		return formattedDesc + (isPrimary?(' ' + i18nModel.getLabel('label.instructor.Primary') + ' ' ) : '');
	},
	
	getFormattedDesc : function(description, id){
		return (description ? description : id);
	},
	
	getSegmentTimings : function(startDateDisplay, endDateDisplay){
		var i18nModel = this.getComponent().getModel('i18nModel');
		return i18nModel.getLabel('label.Timings', [startDateDisplay, endDateDisplay] );
	},
	
	_getSegmentKey: function (scheduleID, segmentID) {
		return scheduleID + "_" + segmentID;
	},
	
	isInstructorsPanelVisible: function(iCount){
		return (iCount>0);
	},
	
	isEquipmentsPanelVisible: function(iCount){
		return (iCount>0);
	},
	
	isInstructorsPhotoVisble: function(isUserPhotosHideGlobally){
		return (isUserPhotosHideGlobally == false);
	},
	
	getInstructorImageUrl: function(imageUrl){
		var appParamsModelData = this.getComponent().getModel('appParamsModel').getData();
		return (imageUrl ? imageUrl : appParamsModelData.contextPath + "/ui/common/img/defaultIcons/photoNotAvailable.gif");
	},
	
	getInstructorName: function(instructorFName, instructorMI, instructorLName, instructorId, isPrimary) {
		var formattedInstuctorName;
		var i18nModel = this.getComponent().getModel('i18nModel');
		if(!instructorFName && !instructorMI && !instructorLName){
			formattedInstuctorName = instructorId;
		}else {
			formattedInstuctorName = i18nModel.getLabel("format.FirstName.MI.LastName", [(instructorFName ? instructorFName : ''), (instructorMI ? instructorMI : ''), (instructorLName ? instructorLName : '')]);
		}
		return formattedInstuctorName + (isPrimary?(' ' + i18nModel.getLabel('label.instructor.Primary') + ' ' ) : '');
	},
	
	onNavBack: function (oEvent) {
		this.getRouter().myNavBack("SO_DETAILS", {
			scheduleID: this._scheduleID 
		});
	},
		
	getComponent: function () {
		return sap.ui.component(sap.ui.core.Component.getOwnerIdFor(this.getView()));
	},
	
	getRouter: function () {
		return sap.ui.core.UIComponent.getRouterFor(this);
	},
	
	getEventBus : function() {
		return sap.ui.getCore().getEventBus();
	}
});